const co = require('co');
const platform = require('../config/platform');
const ReportConfig = require('../config/report');
const _ = require('lodash');
const moment = require('moment');
const ReportData = require('../model/ReportData');
const CacheData = require('../model/CacheData');

//
function objectPlus(o1, o2) {
    const res = {};
    Object.keys(o1).forEach(key => {
        if (key == 'date') {
            res[key] = o2.date;
        } else {
            res[key] = parseInt(o1[key]) + parseInt(o2[key]);
        }
    });
    return res;
}

// 给一个数组加上另外一个数组的数据，2个数组中的对象格式一样
function plusDateData(baseList, plusList) {
    plusList.forEach(item => {
        const index = _.findIndex(baseList, (o) => {
            return o.date == item.date;
        });
        if (index > -1) {
            baseList[index] = objectPlus(baseList[index], item);
        } else {
            baseList.push(item);
        }
    });
    return baseList;
}

/**
 * 根据最近几天的数据计算时间区域
 * @param lastDays
 * @returns {'begin_date':beginDate,'end_date':endDate}
 */
function changeLastDays(lastDays) {
    const endDate = moment().subtract(1, 'days').format('YYYY-MM-DD');
    const beginDate = moment()
        .subtract(lastDays - 1, 'days')
        .format('YYYY-MM-DD');
    return {
        beginDate,
        endDate
    };
}

const Ctrl = {
    // 获取单个账号的报表数据
    getOneAccountReport: function* (ctx, account, beginDate, endDate) {
        const { app } = ctx;
        const platfromConfig = platform[account.type];
        account.uid = account.uid.toString();
        const dbData = yield ReportData.findOne({
            accountId: account.uid,
            beginDate,
            endDate
        });
        // 使用缓存时
        if (dbData && !ctx.message.noCache) {
            console.log('report data from db:', dbData);
            return dbData.toJSON();
        }
        const report = yield app.PlatformApi[platfromConfig.apiName].getHomeReportData(account.cookiesMap, {
            begin_date: beginDate,
            end_date: endDate,
            token: account.token,
            uid: account.uid,
            accountData: account
        });
        if (!dbData) {
            // 添加缓存
            const cache = ReportData.create({
                beginDate,
                endDate,
                type: account.type,
                dataList: report,
                accountId: account.uid,
                updateTime: new Date()
            });
            yield cache.save();
            return cache.toJSON();
        }
        // 更新缓存
        dbData.dataList = report;
        dbData.type = account.type;
        dbData.updateTime = new Date();
        yield dbData.save();
        return dbData.toJSON();
    },
    getExtendOverviewData: function* (ctx, account) {
        const { app } = ctx;
        const platfromConfig = platform[account.type];
        const cacheId = `overview_${account.uid}`;
        const dbData = yield CacheData.findOne({
            cacheId
        });
        // 使用缓存时
        if (dbData && !ctx.message.noCache) {
            console.log('overview data from db:', dbData);
            return dbData.data;
        }
        const data = yield app.PlatformApi[platfromConfig.apiName].getHomeOverviewExtend(account);
        if (!dbData) {
            // 添加缓存
            const cache = CacheData.create({
                cacheId,
                data
            });
            yield cache.save();
            return data;
        }
        // 更新缓存
        dbData.data = data;
        yield dbData.save();
        return data;
    },
    preCheck: co.wrap(function*(ctx, next) {
        const { app, message } = ctx;
        const accoutRes = yield app.AccountService.getAccountList(message.type);
        ctx.dateRange = changeLastDays(message.days);
        if (!accoutRes.rows.length) {
            this.body = {
                overview: [],
                list: [],
                msg: '还没有账号',
                code: 400
            };
            return;
        }
        ctx.accountList = accoutRes.rows.map(item => {
            return item.doc.data;
        });
        ctx.reportConfig = ReportConfig[message.type];
        ctx.platfromConfig = platform[message.type];
        yield next();
    }),
    // 所有账号的数据详情报表
    detail: co.wrap(function*(ctx, next) {
        const { accountList, dateRange } = ctx;
        const allData = [];
        let account;
        for (let i = 0; i < accountList.length; i++) {
            account = accountList[i];
            try {
                const report = yield Ctrl.getOneAccountReport(ctx, account, dateRange.beginDate, dateRange.endDate);
                console.log('on account:', report);
                allData.push({
                    account,
                    dataList: report.dataList,
                    updateTime: report.updateTime
                });
            } catch (e) {
                allData.push({
                    account,
                    message: e.message || e,
                    updateTime: new Date()
                });
            }
        }
        ctx.body.detailList = allData;
        ctx.body.detailListLabel = ctx.reportConfig.detailDataLabel;
        yield next();
    }),
    // 某个平台所有账号的报表数据总和
    all: co.wrap(function*(ctx, next) {
        // const { accountList, dateRange } = ctx;
        // const allData = [];
        // let account;
        // for (let i = 0; i < accountList.length; i++) {
        //     account = accountList[i];
        //     try {
        //         const report = yield Ctrl.getOneAccountReport(ctx, account, dateRange.beginDate, dateRange.endDate);
        //         console.log('on account:', report);
        //         allData.push(report.dataList);
        //     } catch (e) {
        //         console.log(e);
        //     }
        // }
        // 累加数据, 从前面一取到的detailList中累加
        const allData = ctx.body.detailList;
        const dataList = [];
        allData.forEach((item) => {
            if (!item.message) {
                plusDateData(dataList, item.dataList);
            }
        });
        console.log('all data::', dataList);
        ctx.body.allList = dataList;
        // overview数据
        if (dataList.length) {
            ctx.body.overview = dataList[dataList.length - 1];
            delete ctx.body.overview.date;
        } else {
            ctx.body.overview = {};
        }
        ctx.body.overviewLabel = ctx.reportConfig.currentDataLabel;
        ctx.body.allListLabel = ctx.reportConfig.detailDataLabel;
        yield next();
    }),
    removeCache: co.wrap(function*(ctx, next) {
        const removeIds = ctx.message.ids;
        for (let i = 0; i < removeIds.length; i++) {
            const id = removeIds[i];
            // 清空这个账号的所有缓存数据
            yield ReportData.deleteMany({
                accountId: id
            });
            yield CacheData.deleteMany({
                cacheId: `overview_${id}`
            });
        }
        yield next();
    }),
    // 扩展报表overview数据
    extend: co.wrap(function*(ctx, next) {
        const { body, reportConfig, accountList, message } = ctx;
        if (reportConfig.overviewExtendLabel && message.type == 'zhihu') {
            const extendData = {};
            let account;
            for (let i = 0; i < accountList.length; i++) {
                account = accountList[i];
                try {
                    const data = yield Ctrl.getExtendOverviewData(ctx, account);
                    if (data) {
                        Object.keys(data).forEach(key => {
                            extendData[key] = (extendData[key] || 0) + data[key];
                        });
                    }
                } catch (e) {
                    console.log(e);
                }
            }
            body.overview = Object.assign(body.overview, extendData);
            body.overviewLabel = Object.assign(body.overviewLabel, reportConfig.overviewExtendLabel);
        }
        console.log(body);
        yield next();
    })
};

module.exports = Ctrl;
